import fs from 'fs';
import es from 'event-stream';
import elasticsearch from 'elasticsearch';

function createMappingFactory(ref) {
  var client = ref.client;
  var indexName = ref.indexName;
  var mappings = ref.mappings;
  var verbose = ref.verbose;

  return function () { return (new Promise(function (resolve, reject) {
    if (
      typeof mappings === 'object' &&
      mappings !== null
    ) {
      client.indices.create({
        index: indexName,
        body: {
          mappings: mappings
        }
      }, function (err, resp) {
        if (err) {
          console.log('Error creating mapping', err);
          reject();
          return;
        }
        verbose && console.log('Created mapping', resp);
        resolve();
      });
    } else {
      resolve();
    }
  })); };
}

var EventEmitter = require('events');

var queueEmitter = new EventEmitter();

// a simple helper queue to bulk index documents
function indexQueueFactory(ref) {
  var client = ref.client;
  var indexName = ref.indexName;
  var typeName = ref.typeName; if ( typeName === void 0 ) typeName = 'doc';
  var bufferSize = ref.bufferSize; if ( bufferSize === void 0 ) bufferSize = 1000;
  var verbose = ref.verbose; if ( verbose === void 0 ) verbose = true;

  var buffer = [];
  var queue = [];
  var ingesting = false;

  var ingest = function (b) {
    if (typeof b !== 'undefined') {
      queue.push(b);
    }

    if (ingesting === false) {
      var docs = queue.shift();
      ingesting = true;
      verbose && console.log(("bulk ingest docs: " + (docs.length) + ", queue length: " + (queue.length)));

      client.bulk({
        body: docs
      }, function (err, resp) {
        //console.log('error', err, resp);
        ingesting = false;
        if (queue.length > 0) {
          ingest();
        }
      });

    }

    //console.log(`ingest: queue.length ${queue.length}`);
    if (queue.length === 0) {
      queueEmitter.emit('resume');
    }

    return [];
  };

  return {
    add: function (doc, cb) {
      if ( cb === void 0 ) cb = function() {};

      var header = { index: { _index: indexName, _type: typeName } };
      buffer.push(header);
      buffer.push(doc);

      //console.log(`add: queue.length ${queue.length}`);
      if (queue.length === 0) {
        queueEmitter.emit('resume');
      }

      if (buffer.length >= bufferSize) {
        buffer = ingest(buffer);
      }

    },
    finish: function () {
      buffer = ingest(buffer);
    },
    queueEmitter: queueEmitter
  };
}

function transformer(ref) {
	var deleteIndex = ref.deleteIndex; if ( deleteIndex === void 0 ) deleteIndex = false;
	var host = ref.host; if ( host === void 0 ) host = 'localhost';
	var port = ref.port; if ( port === void 0 ) port = '9200';
	var bufferSize = ref.bufferSize; if ( bufferSize === void 0 ) bufferSize = 1000;
	var fileName = ref.fileName;
	var indexName = ref.indexName;
	var typeName = ref.typeName;
	var mappings = ref.mappings;
	var transform = ref.transform;
	var verbose = ref.verbose; if ( verbose === void 0 ) verbose = true;


	var client = new elasticsearch.Client({
		host: (host + ":" + port)
	});

	var createMapping = createMappingFactory({ client: client, indexName: indexName, mappings: mappings, verbose: verbose });
	var indexer = indexQueueFactory({ client: client, indexName: indexName, typeName: typeName, bufferSize: bufferSize, verbose: verbose });

	client.indices.exists({
		index: indexName
	}, function (err, resp) {
		if (resp === false) {
			createMapping().then(indexFile);
		} else {
			if (deleteIndex === true) {
				client.indices.delete({
					index: indexName
				}, function (err, resp) {
					createMapping().then(indexFile);
				});
			} else {
				indexFile();
			}
		}
	});

	function indexFile() {
		var s = fs.createReadStream(fileName)
			.pipe(es.split())
			.pipe(es.mapSync(function (line) {
				s.pause();
				try {
					var doc = (typeof transform === 'function') ? transform(line) : line;

					if (typeof doc === 'undefined') {
						//console.log('doc invalid');
						s.resume();
						return undefined;
					}

					indexer.add(doc);
				} catch (e) {
					console.log('error', e);
				}

			})
				.on('error', function (err) {
					console.log('Error while reading file.', err);
				})
				.on('end', function () {
					verbose && console.log('Read entire file.');
					indexer.finish();
				})
			);

		indexer.queueEmitter.on('resume', function () {
			s.resume();
		});
	}
}

export { transformer };
